# %%
import matplotlib as mpl
import numpy as np
import inspiral_range as ir

from matplotlib import pyplot as plt
from matplotlib.backends.backend_pdf import PdfPages
from scipy.interpolate import interp1d
from scipy.integrate import cumtrapz

# %%
mpl.rcParams.update({
    'text.usetex': True,
    'text.latex.preamble': r'\usepackage{amsmath,lxfonts,upgreek}'#\usepackage{newtxtext,newtxmath}',
    })

def makegrid(ax, color='k', alpha=0.075, alpha_minor=None, **kwargs):
    if alpha_minor is None:
        alpha_minor = 2 / 3 * alpha
    ax.grid(True, which='major', color=color, alpha=alpha, **kwargs)
    ax.grid(True, which='minor', color=color, alpha=alpha_minor, **kwargs)

text_bbox = dict(fc='w', alpha=0.7, lw=0, pad=0.1)

# %%
f, c1, c2, c3 = np.loadtxt('dcpd_xcorr_100Hz.txt', unpack=1)
_, s1, s2, s3 = np.loadtxt('dcpd_sum_100Hz.txt', unpack=1)
_, g1, g2, g3 = np.loadtxt('dcpd_imcwfs_coh_100Hz.txt', unpack=1)
m1 = np.all([f>80, f<=95], axis=0)
m2 = np.all([f>95, f<=115], axis=0)
m3 = np.all([f>115, f<155], axis=0)

f_all = np.concatenate([
    f[m1],
    f[m2],
    f[m3],
])

c_all = np.concatenate([
    c1[m1],
    c2[m2],
    c3[m3],
])

s_all = np.concatenate([
    s1[m1],
    s2[m2],
    s3[m3],
])
g_all = np.concatenate([
    g1[m1],
    g2[m2],
    g3[m3],
])

g_mask = g_all > 0.03
g_all_m = g_all * g_mask

f_therm, a_therm = np.loadtxt('aligoNB_thermal_total.txt', unpack=1)

#import gwinc.io

#traces_nb = gwinc.io.load_hdf5('lho_all_noisebudgets.hdf5')
#f_therm = traces_nb.freq
#a_therm = traces.Thermal.asd

cal = 5.5e12 # mA/m
# %%
fig_x = plt.figure(figsize=(5, 7))
gs_x = fig_x.add_gridspec(2, 1, height_ratios=(3, 1.25))
ax_x = fig_x.add_subplot(gs_x[0])
ax_x.set_facecolor([1, 1, 1, 0])
ax_q = fig_x.add_subplot(gs_x[1], sharex=ax_x)
ax_strain = ax_x.twinx()

ax_x.plot(
    f_all,
    s_all / cal,
    color='xkcd:strawberry',
    lw=2,
    #label=r'A + B',
    label=r'$\sqrt{\langle I_{\text{A+B}}^* I_{\text{A+B}} \rangle}$',
    zorder=100,
    )
#blah = s_all**2 - 8.0e-8**2 - 4 * c_all
#ax_x.plot(
#    f_all,
#    np.sqrt(np.abs(blah)) * np.sign(blah) / cal,
#    color='xkcd:black',
#    lw=2,
#    #label=r'A $\times$ B',
#    label=r'Residual',
#    zorder=90,
#    )
ax_x.plot(
    f_all,
    np.sqrt(4 * c_all) / cal,
    color='xkcd:dark plum',
    lw=2,
    #label=r'A $\times$ B',
    label=r'$\sqrt{4 \langle I_\text{A}^* \, I_\text{B}\rangle}$',
    zorder=90,
    )
ax_x.plot(
    f_all,
    np.sqrt(s_all**2 - 8.0e-8**2) / cal,
    color='xkcd:vibrant green',
    alpha=0.8,
    lw=3,
    #ls=(0, (2, 2,)),
    #label=r'A + B',
    label=r'$\sqrt{\langle I_{\text{A+B}}^* I_{\text{A+B}} \rangle - S_\text{shot}(20\,\text{mA})}$',
    zorder=70,
    )
ax_x.plot(
    f_therm,
    a_therm,
    color='xkcd:electric blue',
    lw=3,
    label=r'Total expected $\sqrt{S_\text{thermal}}$',
    zorder=80,
    )
#ax_x.plot(
#    f_therm,
#    1.75e-20 * (f_therm / 100)**-0.45,
#    '--',
#    color='xkcd:gray',
#    lw=3,
#    #label=r'Total expected $\sqrt{S_\text{thermal}}$',
#    zorder=200,
#    )

a_therm_approx = lambda f: 1.128e-20 * (f/100)**-0.455

ax_q.plot(
    f_all,
    np.sqrt(4 * c_all / cal**2 / a_therm_approx(f_all)**2),
    color='xkcd:pumpkin orange',
    lw=2,
    #label=r'A $\times$ B',
    label=r'$\sqrt{4 \langle I_\text{A}^* \, I_\text{B}\rangle / S_\text{thermal}}$',
    zorder=100,
    )
ax_x.set_xlim([80, 155])
ymin, ymax = 0e-20, 2.7e-20
ax_x.set_ylim([ymin, ymax])

strain_color = 'xkcd:teal'
ax_strain.set_ylim([ymin / 3995, ymax / 3995])
ax_strain.tick_params(axis='y', color=strain_color, labelcolor=strain_color)
ax_x.spines['right'].set_color(strain_color)
#ax_strain.spines['right'].set_zorder(2)

ax_q.set_ylim(1.2, 1.9)

ax_q.yaxis.set_major_locator(mpl.ticker.MultipleLocator(0.1))

ax_q.set_xlabel('Frequency [Hz]')
ax_x.set_ylabel(
    r'Displacement noise $\bigl[\text{m}\big/\!\sqrt{\text{Hz}}\bigr]$'
)
ax_strain.set_ylabel(
    r'Strain noise $\bigl[1\big/\!\sqrt{\text{Hz}}\bigr]$',
    rotation=270,
    va='bottom',
    color=strain_color,
)
ax_q.set_ylabel(
    r'Noise ratio'
)

makegrid(ax_x)
makegrid(
    ax_strain,
    color=strain_color,
    alpha=0.15,
    ls=(0,(4, 2,)),
    )#, zorder=-1)
makegrid(ax_q)

ax_x.set_title('Hanford, 360\,kW, no squeezing')

#lines = [l for l in ax_x.get_lines()] + [l for l in ax_q.get_lines()]
#labels = [l.get_label() for l in lns]
#ax_x.legend(lns, labs, loc='lower left')

def make_legends():
    ax_x.legend(
        loc='lower left',
        #fontsize='small',
        )
    ax_x.set_zorder(1)
    ax_q.legend(loc='upper left', bbox_to_anchor=(0, 1.2), framealpha=0.9).set_zorder(1000)

make_legends()
fig_x.savefig('dcpd_xcorr_nosqz_2023-04-06.pdf', bbox_inches='tight')

ax_x.plot(
    f_all,
    np.sqrt(g_all_m * s_all**2) / cal,
    color='xkcd:cinnamon',
    lw=3,
    label=r'Estimated $\sqrt{S_\text{jitter}}$',
    zorder=1,
    )
ax_q.plot(
    f_all,
    np.sqrt((4 * c_all - g_all * s_all**2) / cal**2 / a_therm_approx(f_all)**2),
    color='xkcd:black',
    lw=2,
    #label=r'A $\times$ B',
    label=r'$\sqrt{(4 \langle I_\text{A}^* \, I_\text{B}\rangle - S_\text{jitter}) / S_\text{thermal}}$',
    zorder=120,
    )

make_legends()
fig_x.savefig('dcpd_xcorr_nosqz_jitter_2023-04-06.pdf', bbox_inches='tight')

# %%
import gwinc.io

traces_nb = gwinc.io.load_hdf5('lho_all_noisebudgets.hdf5')
fig_nb, ax_nb = plt.subplots()
traces_nb.Thermal.plot(ax=ax_nb)
ax_nb.set_xscale('linear')
ax_nb.set_yscale('linear')

ax_nb.set_xlim([80, 155])
ax_nb.set_ylim([0, 1.5e-20])

ax_nb.set_ylabel(
    r'Displacement noise $\bigl[\text{m}\big/\!\sqrt{\text{Hz}}\bigr]$'
)

fig_nb.savefig('aligoNB_H1.pdf', bbox_inches='tight')
# %%
